import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { ethers } from 'ethers';

// ABI du contrat - à remplacer par le véritable ABI après compilation
const VOTING_ABI = [
  // Fonctions et événements du contrat Voting
  "function owner() view returns (address)",
  "function workflowStatus() view returns (uint8)",
  "function getVoter(address _address) view returns (bool isRegistered, bool hasVoted, uint votedProposalId)",
  "function getProposal(uint _id) view returns (string description, uint voteCount)",
  "function getProposalsCount() view returns (uint)",
  "function registerVoter(address _voterAddress)",
  "function startProposalsRegistration(uint _durationInMinutes)",
  "function endProposalsRegistration()",
  "function registerProposal(string calldata _description)",
  "function startVotingSession(uint _durationInMinutes)",
  "function endVotingSession()",
  "function vote(uint _proposalId)",
  "function delegateVoteTo(address _to)",
  "function tallyVotes()",
  "event VoterRegistered(address voterAddress)",
  "event WorkflowStatusChange(uint8 previousStatus, uint8 newStatus)",
  "event ProposalRegistered(uint proposalId)",
  "event Voted(address voter, uint proposalId)"
];

@Injectable({
  providedIn: 'root'
})
export class ContractService {
  // Adresse déployée du contrat - à remplacer par l'adresse réelle après déploiement
  private contractAddress = '';
  
  // Provider et Signer ethers
  private provider: ethers.providers.Web3Provider | null = null;
  private signer: ethers.Signer | null = null;
  private contract: ethers.Contract | null = null;
  
  // Observables pour l'état du contrat
  private accountSubject = new BehaviorSubject<string | null>(null);
  account$ = this.accountSubject.asObservable();
  
  private isOwnerSubject = new BehaviorSubject<boolean>(false);
  isOwner$ = this.isOwnerSubject.asObservable();
  
  private isVoterSubject = new BehaviorSubject<boolean>(false);
  isVoter$ = this.isVoterSubject.asObservable();
  
  private currentStatusSubject = new BehaviorSubject<string>('0');
  currentStatus$ = this.currentStatusSubject.asObservable();
  
  private loadingSubject = new BehaviorSubject<boolean>(false);
  loading$ = this.loadingSubject.asObservable();
  
  private errorSubject = new BehaviorSubject<string | null>(null);
  error$ = this.errorSubject.asObservable();
  
  constructor() {
    // Vérifier si MetaMask est disponible
    if (window.ethereum) {
      // Écouter les changements de compte
      window.ethereum.on('accountsChanged', (accounts: string[]) => {
        if (accounts.length > 0) {
          this.accountSubject.next(accounts[0]);
          this.initializeContract();
        } else {
          this.resetState();
        }
      });
    }
  }
  
  // Connexion au portefeuille MetaMask
  async connectWallet(): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!window.ethereum) {
        throw new Error("MetaMask n'est pas installé. Veuillez installer l'extension MetaMask.");
      }
      
      // Demander l'accès aux comptes
      const provider = new ethers.providers.Web3Provider(window.ethereum);
      const accounts = await provider.send('eth_requestAccounts', []);
      
      if (accounts.length === 0) {
        throw new Error("Aucun compte n'a été autorisé.");
      }
      
      this.provider = provider;
      this.signer = provider.getSigner();
      this.accountSubject.next(accounts[0]);
      
      await this.initializeContract();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de la connexion au portefeuille");
      this.loadingSubject.next(false);
    }
  }
  
  // Initialiser le contrat et récupérer les données
  private async initializeContract(): Promise<void> {
    try {
      if (!this.provider || !this.signer || !this.contractAddress) {
        return;
      }
      
      // Créer une instance du contrat
      this.contract = new ethers.Contract(this.contractAddress, VOTING_ABI, this.signer);
      
      // Récupérer le propriétaire du contrat
      const owner = await this.contract.owner();
      const account = this.accountSubject.getValue();
      
      this.isOwnerSubject.next(account?.toLowerCase() === owner.toLowerCase());
      
      // Récupérer le statut actuel du workflow
      const status = await this.contract.workflowStatus();
      this.currentStatusSubject.next(status.toString());
      
      // Vérifier si l'utilisateur est un électeur inscrit
      try {
        if (account) {
          const voter = await this.contract.getVoter(account);
          this.isVoterSubject.next(voter.isRegistered);
        } else {
          this.isVoterSubject.next(false);
        }
      } catch (error) {
        // Si l'erreur est due au fait que l'utilisateur n'est pas un électeur
        this.isVoterSubject.next(false);
      }
      
      // Configurer les écouteurs d'événements
      this.setupEventListeners();
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de l'initialisation du contrat");
    }
  }
  
  // Configurer les écouteurs d'événements pour le contrat
  private setupEventListeners(): void {
    if (!this.contract) return;
    
    // Écouter les changements de statut du workflow
    this.contract.on('WorkflowStatusChange', (previousStatus: number, newStatus: number) => {
      this.currentStatusSubject.next(newStatus.toString());
    });
    
    // Écouter les enregistrements de votants
    this.contract.on('VoterRegistered', async (voterAddress: string) => {
      const account = this.accountSubject.getValue();
      if (account?.toLowerCase() === voterAddress.toLowerCase()) {
        this.isVoterSubject.next(true);
      }
    });
  }
  
  // Réinitialiser l'état quand l'utilisateur se déconnecte
  private resetState(): void {
    this.accountSubject.next(null);
    this.isOwnerSubject.next(false);
    this.isVoterSubject.next(false);
    this.currentStatusSubject.next('0');
    this.provider = null;
    this.signer = null;
    this.contract = null;
  }
  
  // Définir l'adresse du contrat (après déploiement)
  setContractAddress(address: string): void {
    this.contractAddress = address;
    // Réinitialiser le contrat avec la nouvelle adresse
    if (this.signer) {
      this.initializeContract();
    }
  }
  
  // Méthodes pour interagir avec le contrat
  
  // Enregistrer un nouvel électeur
  async registerVoter(voterAddress: string): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.registerVoter(voterAddress);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de l'enregistrement de l'électeur");
      this.loadingSubject.next(false);
    }
  }
  
  // Démarrer la session d'enregistrement des propositions
  async startProposalsRegistration(durationInMinutes: number): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.startProposalsRegistration(durationInMinutes);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors du démarrage de la session d'enregistrement");
      this.loadingSubject.next(false);
    }
  }
  
  // Terminer la session d'enregistrement des propositions
  async endProposalsRegistration(): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.endProposalsRegistration();
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de la clôture de la session d'enregistrement");
      this.loadingSubject.next(false);
    }
  }
  
  // Soumettre une proposition
  async registerProposal(description: string): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.registerProposal(description);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de l'enregistrement de la proposition");
      this.loadingSubject.next(false);
    }
  }
  
  // Démarrer la session de vote
  async startVotingSession(durationInMinutes: number): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.startVotingSession(durationInMinutes);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors du démarrage de la session de vote");
      this.loadingSubject.next(false);
    }
  }
  
  // Terminer la session de vote
  async endVotingSession(): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.endVotingSession();
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de la clôture de la session de vote");
      this.loadingSubject.next(false);
    }
  }
  
  // Voter pour une proposition
  async vote(proposalId: number): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.vote(proposalId);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors du vote");
      this.loadingSubject.next(false);
    }
  }
  
  // Déléguer son vote
  async delegateVote(delegateAddress: string): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.delegateVoteTo(delegateAddress);
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de la délégation du vote");
      this.loadingSubject.next(false);
    }
  }
  
  // Comptabiliser les votes
  async tallyVotes(): Promise<void> {
    try {
      this.loadingSubject.next(true);
      this.errorSubject.next(null);
      
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const tx = await this.contract.tallyVotes();
      await tx.wait();
      
      this.loadingSubject.next(false);
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors du comptage des votes");
      this.loadingSubject.next(false);
    }
  }
  
  // Récupérer les informations sur les propositions
  async getProposals(): Promise<{ id: number; description: string; voteCount: string }[]> {
    try {
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const count = await this.contract.getProposalsCount();
      const proposals = [];
      
      for (let i = 0; i < count; i++) {
        const proposal = await this.contract.getProposal(i);
        proposals.push({
          id: i,
          description: proposal.description,
          voteCount: proposal.voteCount.toString()
        });
      }
      
      return proposals;
    } catch (error: any) {
      this.errorSubject.next(error.message || "Erreur lors de la récupération des propositions");
      return [];
    }
  }
  
  // Récupérer les informations sur un électeur
  async getVoterInfo(address: string): Promise<{ isRegistered: boolean; hasVoted: boolean; votedProposalId: string }> {
    try {
      if (!this.contract) {
        throw new Error('Contrat non initialisé');
      }
      
      const voter = await this.contract.getVoter(address);
      return {
        isRegistered: voter.isRegistered,
        hasVoted: voter.hasVoted,
        votedProposalId: voter.votedProposalId.toString()
      };
    } catch (error) {
      return {
        isRegistered: false,
        hasVoted: false,
        votedProposalId: '0'
      };
    }
  }
}

// Ajouter la définition de l'interface Window pour le provider Ethereum
declare global {
  interface Window {
    ethereum: any;
  }
} 